%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [alpha fk dual DD]=nocsvm(x, hkernel, kernelparam, lambda, alpha)
% function [alpha fk dual DD]=nocsvm(x, hkernel, kernelparam, lambda, alpha)
%	x               N x p data
%	hkernel			kernel function handle
%	kernelparam		kernel parameter
%	lambda          regularization parameter
%	alpha           N x M initial alpha
% 
%   alpha           N x M parameters
%   fk              decision function value ; > 0 in ; < 0 out
%   dual            objective function value
% 
%	nested one class support vector machine
%	find nested decision rules at multiple lambda at the same time
% 
% Gyemin Lee, University of Michigan
% 
tol = 1e-10;

N = size(x,1);
M = length(lambda);
K = hkernel(x,x,kernelparam);
K = (K+K')/2;

loopnum = 0;
maxloopnum = 3000;
bStop = false;
Dprev = Inf;
DD = zeros(maxloopnum,1);
pp_lst = zeros(maxloopnum,1);
err_lst = zeros(maxloopnum,1);

% initialization
if nargin < 5
	alpha = 1/N*ones(N,M);
end

while (~bStop)
	loopnum = loopnum + 1;

    % select a data point that violates the KKT condition most
    gim = K*alpha./repmat(lambda',N,1) - 1;
    [pp err] = selection(lambda, alpha, gim);
    pp_lst(loopnum) = pp;
    err_lst(loopnum) = err;
    
    % solve a subproblem
    anew = alpha(pp,:)' - lambda.*gim(pp,:)'/K(pp,pp);
    alpha(pp,:) = subproblem([N M], lambda, anew, alpha(pp,:)')';
    
    % dual value
	D = sum(diag(alpha'*K*alpha)./(2*lambda)) - sum(alpha(:));
	if (D > Dprev + tol)
        fprintf('WARNING: dual value increased\n');
		alpha = alpha_bak;
		bStop = true;
	else
		if (err < tol)
% 		if (err < tol || abs((D-Dprev)/Dprev) < eps)
% 		if (abs((D - Dprev)/Dprev) < tol || loopnum>=maxloopnum)
			bStop = true;
		end

        Dprev = D;
        DD(loopnum) = D;
        alpha_bak = alpha;
	end
end

fprintf('noc_svm: loop %d \t D: %e \t err: %e\n', loopnum, D, err);

fk = K*alpha./repmat(lambda',N,1) - 1;
dual = sum(diag(alpha'*K*alpha)./(2*lambda)) - sum(alpha(:));
DD = DD(1:loopnum);


function [sel_idx sel_err] = selection(lambda, alpha, gim)

tol = 1e-10;

[N M] = size(alpha);
vim = zeros(N,M);
ferr = zeros(N,M);

mm = M;
idx1 = alpha(:,mm) < 1/N-tol;
idx3 = alpha(:,mm-1)/lambda(mm-1) < alpha(:,mm)/lambda(mm) - tol;

ferr(~idx1,mm) = gim(~idx1,mm).*(gim(~idx1,mm)>0);
vim(  idx1 & ~idx3,mm) = lambda(mm)*gim( idx1 & ~idx3,mm);
ferr( idx1 &  idx3,mm) = abs(gim( idx1 &  idx3,mm));

for mm=M-1:-1:1
    idx1 = alpha(:,mm) < 1/N - tol;
    idx2 = alpha(:,mm)/lambda(mm) < alpha(:,mm+1)/lambda(mm+1) - tol;
    if (mm>1)
        idx3 = alpha(:,mm-1)/lambda(mm-1) < alpha(:,mm)/lambda(mm) - tol;
    else
        idx3 = tol < alpha(:,mm)/lambda(mm);
    end
    
    ferr(~idx1 &  idx2,mm) = gim(~idx1 &  idx2,mm);
    ferr(~idx1 & ~idx2,mm) = gim(~idx1 & ~idx2,mm) + vim(~idx1 & ~idx2,mm+1)/lambda(mm);
    ferr(~idx1,mm) = ferr(~idx1,mm).*(ferr(~idx1,mm)>0);
    
    vim(  idx1 &  idx2 & ~idx3,mm) = lambda(mm)*gim( idx1 &  idx2 & ~idx3,mm);
    vim(  idx1 & ~idx2 & ~idx3,mm) = lambda(mm)*gim( idx1 & ~idx2 & ~idx3,mm) ...
        + vim( idx1 & ~idx2 & ~idx3,mm+1);
    
    ferr( idx1 &  idx2 &  idx3,mm) = abs(gim( idx1 &  idx2 &  idx3,mm));
    ferr( idx1 & ~idx2 &  idx3,mm) = abs(gim( idx1 & ~idx2 &  idx3,mm) ...
        + vim( idx1 & ~idx2 &  idx3,mm+1)/lambda(mm));
end

verrsum = -sum(vim.*(vim<0),2);
ferrsum = sum(ferr,2);

[sel_err sel_idx] = max(verrsum+ferrsum);


function alpha_new = subproblem(size_alpha, lambda, anew, aold)

N = size_alpha(1);
M = size_alpha(2);
% aold = 1/N*ones(M,1);

options = optimset('LargeScale', 'off', 'Display', 'off');
H = diag(1./lambda);
f = -anew./lambda;
A = [zeros(1,M); H(1:M-1,:)] - H(1:M,:);

[alpha_new,D,exitflag] = quadprog(H, f, A, zeros(M,1), [], [], [], ones(M,1)/N, aold, options);
